import R from 'ramda';
import React from 'react';
import { Seed } from 'react-seeds';
import rgba from 'react-sow/rgba';
import repeatString from 'lodash/repeat';

import { renderTreeUsing } from './render';
import multilinedText from './multilinedText';
import divider from './divider';
import JSONComponent from './JSONComponent';
import CollectedImage from './components/CollectedImage';
import UnsplashImage from './components/UnsplashImage';

var whenPresent = R.unless(R.isNil);

import * as Message from './Message';
import * as Swatch from './Swatch';
import * as Distribution from './Distribution';
import * as LayoutCalculator from './LayoutCalculator';

export var isPassword = function isPassword(tags, mentions, title) {
	return R.has('password', tags) || R.test(/\bpassword\b/i, title);
};

export var field = function field(tags, mentions, title, children, Element, resolveContent) {
	var value = '';
	var onChange = void 0;
	if (R.has('value', tags)) {
		value = resolveContent(tags.value);
		/*value = R.unless(
  	R.is(String),
  	R.always(null),
  	resolveContent(tags.value)
  )*/
		onChange = function onChange(event) {
			var newValue = event.target.value;
			resolveContent(tags.value, { alter: R.always(newValue) });
		};
	}

	return React.createElement(
		Seed,
		{ Component: 'label', column: true,
			alignSelf: 'center'
		},
		React.createElement('span', { key: 'label', children: title, style: { display: 'block' } }),
		React.createElement('input', { key: 'input',
			type: isPassword(tags, title) ? 'password' : 'text',
			value: value,
			onChange: onChange
		})
	);
};

export var button = function button(tags, mentions, title) {
	return React.createElement(Seed, { Component: 'button',
		shrink: 0, alignSelf: 'center',
		margin: { base: 'auto', bottom: '0.5rem' },
		maxWidth: '20em',
		children: title
	});
};
export var cta = button;

export var choice = function choice(tags, mentions, title, children, Element, resolveContent) {
	return React.createElement(
		Seed,
		{ Component: 'label', column: true },
		React.createElement('span', { children: title, style: { display: 'block' } }),
		React.createElement(
			Seed,
			{ Component: 'select',
				value: tags.value
				//onChange={ this.onChange }
				, shrink: 0,
				maxWidth: '20em'
				//padding={{ top: 0, bottom: 0, left: '0.5em', right: '0.5em' }}
				, font: { size: 16 }
			},
			children.map(function (_ref) {
				var text = _ref.text,
				    tags = _ref.tags;
				return React.createElement('option', { key: text,
					value: text, children: tags.title || text
				});
			})
		)
	);
};

var wrapForTags = function wrapForTags(tags, resolveContent, element) {
	if (R.has('link', tags)) {
		var url = resolveContent(tags.link);
		return React.createElement('a', { href: url, rel: 'nofollow', children: element });
	} else {
		return element;
	}
};

var wrapInInline = R.curry(function (tags, element) {
	if (R.has('small', tags)) {
		element = React.createElement('small', { children: element });
	}

	return element;
});

export var text = function text(tags, references, content, children, Element, resolveContent) {
	var _ref2 = R.has('primary', tags) ? ['h1', '2em', 'center'] : R.has('secondary', tags) ? ['h2', '1.5em', 'center'] : R.has('tertiary', tags) ? ['h3', '1.25em', 'left'] : R.has('quote', tags) ? ['blockquote', '1em', 'left'] : ['p', '1em', 'left'],
	    Component = _ref2[0],
	    fontSize = _ref2[1],
	    textAlign = _ref2[2];

	return wrapForTags(tags, resolveContent, React.createElement(Seed, {
		maxWidth: '40rem',
		alignSelf: 'center',
		margin: 'auto',
		text: { align: textAlign },
		font: { size: fontSize },
		children: multilinedText(resolveContent({ references: references, text: content }), Component, wrapInInline(tags))
	}));
};

var imageHeight = 150;
var imageBackground = { color: rgba.whiteValue(0, 0.1) };

var placeholderImageContent = function placeholderImageContent(tags, text, resolveContent) {
	return React.createElement(Seed, { column: true, justifyContent: 'center',
		minHeight: imageHeight,
		font: { style: 'italic' },
		background: imageBackground,
		children: text
	});
};

var richImageContent = function richImageContent(tags, text, resolveContent) {
	if (R.has('unsplash', tags)) {
		return React.createElement(UnsplashImage, {
			category: resolveContent(tags['unsplash']),
			text: text
		});
	} else if (R.has('collected', tags)) {
		return React.createElement(CollectedImage, {
			sha256: resolveContent(tags['collected']),
			width: whenPresent(resolveContent, tags['width']),
			height: whenPresent(resolveContent, tags['height']),
			text: text
		});
	} else {
		return placeholderImageContent(tags, text, resolveContent);
	}
};

export var imageMaker = function imageMaker(imageContent) {
	return function (tags, mentions, text, children, Element, resolveContent) {
		return React.createElement(Seed, { Component: 'figure', column: true,
			grow: 1, width: '100%',
			text: { align: 'center' },
			children: wrapForTags(tags, resolveContent, R.has('nocaption', tags) ? imageContent(tags, text, resolveContent) : [imageContent(tags, null, resolveContent), React.createElement(Seed, { key: 'caption', Component: 'figcaption',
				text: { lineHeight: '1.3' },
				font: { style: 'italic' },
				children: text
			})])
		});
	};
};

export var image = imageMaker(richImageContent);
export var placeholderImage = imageMaker(placeholderImageContent);

export var video = function video(tags, mentions, content) {
	return React.createElement(Seed, { Component: 'figure', column: true,
		grow: 1, alignItems: 'center', justifyContent: 'center',
		width: '100%', height: 0,
		padding: { top: 9 / 16 / 2 * 100 + '%', bottom: 9 / 16 / 2 * 100 + '%' },
		background: { color: rgba.whiteValue(0, 0.1) },
		children: '\u25B6'
	});
};

export var code = function code(tags, references, text, children, Element, resolveContent) {
	return React.createElement(
		Seed,
		{ Component: 'pre',
			maxWidth: '100%',
			overflow: 'scroll'
		},
		React.createElement(Seed, { Component: 'code',
			children: resolveContent({ references: references, text: text })
		})
	);
};

var isHiddenForTags = R.propSatisfies(Boolean, 'hidden');

export var hidden = function hidden(tags, mentions, content) {
	return React.createElement('noscript', null);
};

export var list = function list(tags, mentions, content, children, Element, resolveContent) {
	var Component = R.propEq('ordered', true, tags) ? 'ol' : 'ul';
	return React.createElement(
		Seed,
		null,
		text(tags, mentions, content || '', children, Element, resolveContent),
		React.createElement(
			Seed,
			{ Component: Component

			},
			children.map(function (element, index) {
				return React.createElement(
					'li',
					{ key: index },
					React.createElement(Element, element)
				);
			})
		)
	);
};

var columnsComponentForTags = R.cond([[R.has('nav'), R.always('nav')], [R.has('figure'), R.always('figure')], [R.T, R.always('div')]]);

export var columns = function columns(tags, mentions, content, children, Element, resolveContent) {
	var columnWidth = R.ifElse(R.gt(R.__, 0), // Greater than 0
	function (number) {
		return 100.0 / number + '%';
	}, R.always(null))(parseInt(resolveContent(tags.columns), 10));

	var renderColumn = function renderColumn(props) {
		return React.createElement(
			'div',
			{ style: { flexBasis: columnWidth } },
			Element(props)
		);
	};

	return React.createElement(
		Seed,
		{ Component: columnsComponentForTags(tags),
			row: true, wrap: true, reverse: R.has('reverse', tags), justifyContent: 'center',
			width: '100%'
		},
		// Render content, interleaving an optional divider
		R.pipe(R.converge(R.concat, [R.pipe(R.init, R.map(renderColumn)), R.pipe(R.last, R.ifElse(R.isNil, R.always([]), renderColumn))]))(children)
	);
};

export var nav = columns;

export var record = function record(tags, mentions, text, children, Element, resolveContent) {
	if (mentions.length > 0) {
		return React.createElement(JSONComponent, { isDeserialized: true, json: mentions[0] });
	} else {
		return React.createElement(JSONComponent, { isDeserialized: true, json: text });
	}
};

// import Frame from 'react-frame-component'

// export const gist = (tags, references, text, children, Element, resolveContent) => (
// 	<Frame head={ <script src={ resolveContent({ references, text }) + '.js' } /> } />
// )

export var extendTagConds = function extendTagConds(conds) {
	return R.pipe(Message.useWithFallback, Swatch.useWithFallback, Distribution.useWithFallback, LayoutCalculator.useWithFallback)(R.cond([[isHiddenForTags, R.curry(hidden)]].concat(conds, [[R.has('field'), R.curry(field)], [R.has('button'), R.curry(button)], [R.has('cta'), R.curry(cta)], [R.has('choice'), R.curry(choice)], [R.has('image'), R.curry(image)], [R.has('video'), R.curry(video)], [R.has('code'), R.curry(code)], [R.has('columns'), R.curry(columns)], [R.has('nav'), R.curry(nav)], [R.has('record'), R.curry(record)],
	//[ R.has('gist'), R.curry(gist) ],
	[R.T, R.curry(list)]])));
};

var elementRendererForTags = extendTagConds([]);

export var Preview = renderTreeUsing({
	elementRendererForTags: elementRendererForTags
});

export function init() {}

export var title = 'Vanilla Web';

export function head() {
	return React.createElement(
		'head',
		null,
		React.createElement('style', { children: '\nhtml {\n\theight: 100%;\n\tbackground-color: #fdfdfd;\n}\nhtml, h1, input, button {\n\tfont-family: \'Lato\', sans-serif;\n}\nbody {\n\theight: 100%;\n\tmargin: 0;\n}\n' })
	);
}

export function deinit() {}